include "Vram.inc"
include "VVram.inc"

ext CharPattern_, SpritePattern_
ext VVram_

VramTop equ Vram+VramRowSize

dseg
Backup: public Backup0,Backup1
Backup0:
	defs VVramWidth*VVramHeight
Backup1:
	defs VVramWidth*VVramHeight
VramBank: public VramBank
    defb 0
pBackup: public pBackup
	defw 0

zseg
yCount:
    defb 0
xCount:
    defb 0
pVram:
    defw 0
invertedByte:
    defb 0
dotCount:
    defb 0


cseg
ClearPage:
    ldb #$10
    do
        stb $fd8b ; MMR $b000
        ldx #$b000
        do
            clr ,x+
            cmpx #$c000
        while ne | wend
        incb
        cmpb #$1c
    while ne | wend
rts

; void ClearScreen();
cseg
ClearScreen_: public ClearScreen_
    pshs a,b,x,y
        lda VramBank | anda #not $20 | sta $f430
        bsr ClearPage
        lda VramBank | ora #$20 | sta $f430
        bsr ClearPage

        ldx #Backup0
        ldy #VVramWidth*VVramHeight*2
        do
            clr ,x+
            leay -1,y
        while ne | wend

        lda #$20
        sta $f430 | sta VramBank
        ldx #Backup1
        stx pBackup
    puls a,b,x,y
rts


cseg
Put: ;(x, y)
    lda #$10
    do
        sta $fd8b ; MMR b000
        inca | sta $fd8c ; MMR c000
        inca | sta $fd8d ; MMR d000
        inca | sta $fd8e ; MMR e000
        inca
        pshs a
            ldb #CharHeight
            do
                lda ,y+ | sta ,x+
                lda ,y+ | sta ,x
                leax VramWidth-1,x
                decb
            while ne | wend
            leax -VramWidth*CharHeight,x
        puls a
        cmpa #$1c
    while ne | wend
rts


; word Put(word vram, byte c);
cseg
Put_: public Put_
    pshs x,y
        lda #PatternSize
        mul
        addd #CharPattern_
        tfr d,y
        
        ldb VramBank
        andb #not $20
        stb $f430
        pshs b,x,y
            bsr Put
        puls b,x,y
        
        orb #$20
        stb $f430
        bsr Put
        
        ldb VramBank | stb $f430
    puls x,y
    tfr x,d
    addd #2
rts


; void VVramToVram();
cseg
VVramToVram_: public VVramToVram_
    pshs a,b,x,y
        ldx #VramTop
        stx <pVram
        ldx #VVram_
        ldy pBackup
        lda #VVramHeight
        sta <yCount
        do
            lda #VVramWidth
            sta <xCount
            do 
                ldb ,x+
                cmpb ,y
                if ne
                    stb ,y
                    pshs x,y
                        lda #PatternSize
                        mul
                        addd #CharPattern_
                        tfr d,y
                        ldx <pVram
                        bsr Put
                    puls x,y
                endif
                leay 1,y
                pshs x
                    ldx <pVram
                    leax 2,x
                    stx <pVram
                puls x
                dec <xCount
            while ne | wend
            pshs x
                ldx <pVram
                leax VramRowSize-VVramWidth*2,x
                stx <pVram
            puls x
            dec <yCount
        while ne | wend
    puls a,b,x,y
rts


; void SwapVram();
cseg
SwapVram_: public SwapVram_
    pshs a,x
        do
            lda $fd12
            anda #$01
        while eq | wend

        lda VramBank
        anda #$20
        if ne
            lda #$40
            ldx #Backup0
        else
            lda #$20
            ldx #Backup1
        endif
        sta $f430 | sta VramBank
        stx pBackup
    puls a,x
rts


; void EraseBackup(byte x, byte y);
cseg
EraseBackup_: public EraseBackup_
    pshs a,b,x
        pshs a,b
            lsrb
            if cs
                lda #3
            else
                lda #2
            endif
            sta <yCount
            lda #VVramWidth
            mul
            addd #Backup0
            tfr d,x
        puls a,b
        lsra
        if cs
            ldb #3
        else
            ldb #2
        endif
        stb <xCount
        leax a,x
        lda #$ff
        do
            pshs x
                ldb <xCount
                do
                    sta Backup1-Backup0,x
                    sta ,x+
                    decb
                while ne | wend
            puls x
            leax VVramWidth,x
            dec <yCount
        while ne | wend
    puls a,b,x
rts


; void DrawSprite(byte x, byte y, byte pattern);
dseg
DrawSprite_@Param2: public DrawSprite_@Param2
    defb 0
cseg
DrawSprite_: public DrawSprite_
    pshs a,b,x,y
        pshs a
            lda #VramRowSize/4
            mul
            lslb
            rola
            addd #VramTop
            tfr d,y
        puls a
        leay a,y

        clrb
        lda DrawSprite_@Param2
        addd #SpritePattern_
        tfr d,x

        lda #CharHeight*2 | sta <yCount
        do
            lda #4 | sta <xCount
            do
                lda ,x+ | coma | sta <invertedByte
                lda #$10
                do
                    sta $fd8b ; MMR b000
                    inca | sta $fd8c ; MMR c000
                    inca | sta $fd8d ; MMR d000
                    inca | sta $fd8e ; MMR e000
                    inca
                    ldb ,y
                    andb <invertedByte
                    orb ,x+
                    stb ,y
                    cmpa #$1c
                while ne | wend
                leay 1,y
                dec <xCount
            while ne | wend
            leay -4+VramWidth,y
            dec <yCount
        while ne | wend
    puls a,b,x,y
rts
